@@ -1,8 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
 # Copyright 2015 Intel Corporation.
-# Copyright 2015 Isaku Yamahata <isaku.yamahata at intel com>
-#                               <isaku.yamahata at gmail com>
 # All Rights Reserved.
 #
 #
@@ -17,9 +15,6 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-#
-# @author: Isaku Yamahata, Intel Corporation.
-# shamelessly many codes are stolen from gbp simplechain_driver.py
 
 import sys
 import time
@@ -29,9 +24,10 @@ from heatclient import client as heat_client
 from heatclient import exc as heatException
 from keystoneclient.v2_0 import client as ks_client
 from oslo_config import cfg
+from toscaparser.utils import yamlparser
 
-from tacker.common import exceptions
 from tacker.common import log
+from tacker.extensions import vnfm
 from tacker.openstack.common import jsonutils
 from tacker.openstack.common import log as logging
 from tacker.vm.drivers import abstract_driver
@@ -105,6 +101,101 @@ class DeviceHeat(abstract_driver.DeviceAbstractDriver):
                 device_template_dict['mgmt_driver'] = mgmt_driver
         LOG.debug(_('device_template %s'), device_template)
 
+    @log.log
+    def _update_params(self, original, paramvalues, match=False):
+        for key, value in original.iteritems():
+            if not isinstance(value, dict) or 'get_input' not in str(value):
+                pass
+            elif isinstance(value, dict):
+                if not match:
+                    if key in paramvalues and 'param' in paramvalues[key]:
+                        self._update_params(value, paramvalues[key]['param'],
+                                            True)
+                    elif key in paramvalues:
+                        self._update_params(value, paramvalues[key], False)
+                    else:
+                        LOG.debug('Key missing Value: %s', key)
+                        raise vnfm.InputValuesMissing()
+                elif 'get_input' in value:
+                    if value['get_input'] in paramvalues:
+                        original[key] = paramvalues[value['get_input']]
+                    else:
+                        LOG.debug('Key missing Value: %s', key)
+                        raise vnfm.InputValuesMissing()
+                else:
+                    self._update_params(value, paramvalues, True)
+
+    @log.log
+    def _process_parameterized_input(self, dev_attrs, vnfd_dict):
+        param_vattrs_yaml = dev_attrs.pop('param_values', None)
+        if param_vattrs_yaml:
+            try:
+                param_vattrs_dict = yaml.load(param_vattrs_yaml)
+                LOG.debug('param_vattrs_yaml', param_vattrs_dict)
+            except Exception as e:
+                LOG.debug("Not Well Formed: %s", str(e))
+                raise vnfm.ParamYAMLNotWellFormed(
+                    error_msg_details=str(e))
+            else:
+                self._update_params(vnfd_dict, param_vattrs_dict)
+        else:
+            raise vnfm.ParamYAMLInputMissing()
+
+    @log.log
+    def _process_vdu_network_interfaces(self, vdu_id, vdu_dict, properties,
+                                        template_dict):
+        def make_port_dict():
+            port_dict = {
+                'type': 'OS::Neutron::Port',
+                'properties': {
+                    'value_specs': {'port_security_enabled': False}
+                }
+            }
+            port_dict['properties'].setdefault('fixed_ips', [])
+            return port_dict
+
+        def make_mgmt_outputs_dict(port):
+            mgmt_ip = 'mgmt_ip-%s' % vdu_id
+            outputs_dict[mgmt_ip] = {
+                'description': 'management ip address',
+                'value': {
+                    'get_attr': [port, 'fixed_ips',
+                                 0, 'ip_address']
+                }
+            }
+
+        def handle_port_creation(network_param, ip_list=[],
+                                 mgmt_port=False):
+            port = '%s-%s-port' % (vdu_id, network_param['network'])
+            port_dict = make_port_dict()
+            if mgmt_port:
+                make_mgmt_outputs_dict(port)
+            for ip in ip_list:
+                port_dict['properties']['fixed_ips'].append({"ip_address": ip})
+            port_dict['properties'].update(network_param)
+            template_dict['resources'][port] = port_dict
+            return port
+
+        networks_list = []
+        outputs_dict = template_dict['outputs']
+        properties['networks'] = networks_list
+        for network_param in vdu_dict[
+                'network_interfaces'].values():
+            port = None
+            if 'addresses' in network_param:
+                ip_list = network_param.pop('addresses', [])
+                if not isinstance(ip_list, list):
+                    raise vnfm.IPAddrInvalidInput()
+                mgmt_flag = network_param.pop('management', False)
+                port = handle_port_creation(network_param, ip_list, mgmt_flag)
+            if network_param.pop('management', False):
+                port = handle_port_creation(network_param, [], True)
+            if port is not None:
+                network_param = {
+                    'port': {'get_resource': port}
+                }
+            networks_list.append(dict(network_param))
+
     @log.log
     def create(self, plugin, context, device):
         LOG.debug(_('device %s'), device)
@@ -120,7 +211,6 @@ class DeviceHeat(abstract_driver.DeviceAbstractDriver):
 
         # overwrite parameters with given dev_attrs for device creation
         dev_attrs = device['attributes'].copy()
-        config_yaml = dev_attrs.pop('config', None)
         fields.update(dict((key, dev_attrs.pop(key)) for key
                       in ('stack_name', 'template_url', 'template')
                       if key in dev_attrs))
@@ -137,14 +227,20 @@ class DeviceHeat(abstract_driver.DeviceAbstractDriver):
             outputs_dict = {}
             template_dict['outputs'] = outputs_dict
 
-            vnfd_dict = yaml.load(vnfd_yaml)
+            vnfd_dict = yamlparser.simple_ordered_parse(vnfd_yaml)
             LOG.debug('vnfd_dict %s', vnfd_dict)
+
+            if 'get_input' in vnfd_yaml:
+                self._process_parameterized_input(dev_attrs, vnfd_dict)
+
             KEY_LIST = (('description', 'description'),
                         )
             for (key, vnfd_key) in KEY_LIST:
                 if vnfd_key in vnfd_dict:
                     template_dict[key] = vnfd_dict[vnfd_key]
 
+            monitoring_dict = {'vdus': {}}
+
             for vdu_id, vdu_dict in vnfd_dict.get('vdus', {}).items():
                 template_dict.setdefault('resources', {})[vdu_id] = {
                     "type": "OS::Nova::Server"
@@ -157,37 +253,15 @@ class DeviceHeat(abstract_driver.DeviceAbstractDriver):
                 for (key, vdu_key) in KEY_LIST:
                     properties[key] = vdu_dict[vdu_key]
                 if 'network_interfaces' in vdu_dict:
-                    # properties['networks'] = (
-                    #     vdu_dict['network_interfaces'].values())
-                    networks_list = []
-                    properties['networks'] = networks_list
-                    for network_param in vdu_dict[
-                            'network_interfaces'].values():
-                        if network_param.pop('management', False):
-                            mgmt_port = 'mgmt_port-%s' % vdu_id
-                            mgmt_port_dict = {
-                                'type': 'OS::Neutron::Port',
-                                'properties': {
-                                    'value_specs': {
-                                        'port_security_enabled': False
-                                    }
-                                }
-                            }
-                            mgmt_port_dict['properties'].update(network_param)
-                            template_dict['resources'][
-                                mgmt_port] = mgmt_port_dict
-                            network_param = {
-                                'port': {'get_resource': mgmt_port}
-                            }
-                            mgmt_ip = 'mgmt_ip-%s' % vdu_id
-                            outputs_dict[mgmt_ip] = {
-                                'description': 'management ip address',
-                                'value': {
-                                    'get_attr': [mgmt_port, 'fixed_ips',
-                                                 0, 'ip_address']
-                                }
-                            }
-                        networks_list.append(network_param)
+                    self._process_vdu_network_interfaces(vdu_id, vdu_dict,
+                                                         properties,
+                                                         template_dict)
+                if 'user_data' in vdu_dict and 'user_data_format' in vdu_dict:
+                    properties['user_data_format'] = vdu_dict[
+                        'user_data_format']
+                    properties['user_data'] = vdu_dict['user_data']
+                elif 'user_data' in vdu_dict or 'user_data_format' in vdu_dict:
+                    raise vnfm.UserDataFormatNotFound()
                 if ('placement_policy' in vdu_dict and
                     'availability_zone' in vdu_dict['placement_policy']):
                     properties['availability_zone'] = vdu_dict[
@@ -199,31 +273,32 @@ class DeviceHeat(abstract_driver.DeviceAbstractDriver):
                     for key, value in metadata.items():
                         metadata[key] = value[:255]
 
-                # monitoring_policy = vdu_dict.get('monitoring_policy', None)
-                # failure_policy = vdu_dict.get('failure_policy', None)
+                monitoring_policy = vdu_dict.get('monitoring_policy', 'noop')
+                failure_policy = vdu_dict.get('failure_policy', 'noop')
+
+                # Convert the old monitoring specification to the new format
+                # This should be removed after Mitaka
+                if monitoring_policy == 'ping' and failure_policy == 'respawn':
+                    vdu_dict['monitoring_policy'] = {'ping': {
+                                                       'actions': {
+                                                           'failure': 'respawn'
+                                                       }}}
+                    vdu_dict.pop('failure_policy')
+
+                if monitoring_policy != 'noop':
+                    monitoring_dict['vdus'][vdu_id] = \
+                        vdu_dict['monitoring_policy']
 
                 # to pass necessary parameters to plugin upwards.
-                for key in ('monitoring_policy', 'failure_policy',
-                            'service_type'):
+                for key in ('service_type',):
                     if key in vdu_dict:
                         device.setdefault(
-                            'attributes', {})[key] = vdu_dict[key]
+                            'attributes', {})[vdu_id] = jsonutils.dumps(
+                                {key: vdu_dict[key]})
 
-            if config_yaml is not None:
-                config_dict = yaml.load(config_yaml)
-                resources = template_dict.setdefault('resources', {})
-                for vdu_id, vdu_dict in config_dict.get('vdus', {}).items():
-                    if vdu_id not in resources:
-                        continue
-                    config = vdu_dict.get('config', None)
-                    if not config:
-                        continue
-                    properties = resources[vdu_id].setdefault('properties', {})
-                    properties['config_drive'] = True
-                    metadata = properties.setdefault('metadata', {})
-                    metadata.update(config)
-                    for key, value in metadata.items():
-                        metadata[key] = value[:255]
+            if monitoring_dict.keys():
+                device['attributes']['monitoring_policy'] = jsonutils.dumps(
+                                                              monitoring_dict)
 
             heat_template_yaml = yaml.dump(template_dict)
             fields['template'] = heat_template_yaml
@@ -266,7 +341,7 @@ class DeviceHeat(abstract_driver.DeviceAbstractDriver):
             stack_retries = stack_retries - 1
 
         LOG.debug(_('stack status: %(stack)s %(status)s'),
-                  {'stack': stack, 'status': status})
+                  {'stack': str(stack), 'status': status})
         if stack_retries == 0:
             LOG.warn(_("Resource creation is"
                        " not completed within %(wait)s seconds as "
@@ -274,7 +349,7 @@ class DeviceHeat(abstract_driver.DeviceAbstractDriver):
                      {'wait': (STACK_RETRIES * STACK_RETRY_WAIT),
                       'stack': device_id})
         if status != 'CREATE_COMPLETE':
-            raise RuntimeError(_("creation of server %s faild") % device_id)
+            raise vnfm.DeviceCreateWaitFailed(device_id=device_id)
         outputs = stack.outputs
         LOG.debug(_('outputs %s'), outputs)
         PREFIX = 'mgmt_ip-'
@@ -296,7 +371,13 @@ class DeviceHeat(abstract_driver.DeviceAbstractDriver):
         update_yaml = device['device'].get('attributes', {}).get('config', '')
         LOG.debug('yaml orig %(orig)s update %(update)s',
                   {'orig': config_yaml, 'update': update_yaml})
-        config_dict = yaml.load(config_yaml) or {}
+
+        # If config_yaml is None, yaml.load() will raise Attribute Error.
+        # So set config_yaml to {}, if it is None.
+        if not config_yaml:
+            config_dict = {}
+        else:
+            config_dict = yaml.load(config_yaml) or {}
         update_dict = yaml.load(update_yaml)
         if not update_dict:
             return
@@ -405,7 +486,7 @@ class HeatClient:
             return self.stacks.create(**fields)
         except heatException.HTTPException:
             type_, value, tb = sys.exc_info()
-            raise exceptions.HeatClientError(msg=value)
+            raise vnfm.HeatClientException(msg=value)
 
     def delete(self, stack_id):
         try:
