@@ -20,6 +20,7 @@
 #
 # @author: Isaku Yamahata, Intel Corporation.
 
+import copy
 import eventlet
 import inspect
 
@@ -28,10 +29,9 @@ from sqlalchemy.orm import exc as orm_exc
 
 from tacker.api.v1 import attributes
 from tacker.common import driver_manager
-from tacker import context as t_context
 from tacker.db.vm import proxy_db  # noqa
 from tacker.db.vm import vm_db
-from tacker.extensions import servicevm
+from tacker.extensions import vnfm
 from tacker.openstack.common import excutils
 from tacker.openstack.common import log as logging
 from tacker.plugins.common import constants
@@ -41,18 +41,20 @@ from tacker.vm import monitor
 LOG = logging.getLogger(__name__)
 
 
-class ServiceVMMgmtMixin(object):
+class VNFMMgmtMixin(object):
     OPTS = [
         cfg.MultiStrOpt(
             'mgmt_driver', default=[],
             help=_('MGMT driver to communicate with '
                    'Hosting Device/logical service '
                    'instance servicevm plugin will use')),
+        cfg.IntOpt('boot_wait', default=30,
+            help=_('Time interval to wait for VM to boot')),
     ]
     cfg.CONF.register_opts(OPTS, 'servicevm')
 
     def __init__(self):
-        super(ServiceVMMgmtMixin, self).__init__()
+        super(VNFMMgmtMixin, self).__init__()
         self._mgmt_manager = driver_manager.DriverManager(
             'tacker.servicevm.mgmt.drivers', cfg.CONF.servicevm.mgmt_driver)
 
@@ -152,7 +154,7 @@ class ServiceVMMgmtMixin(object):
             service_instance=service_instance_dict, kwargs=kwargs)
 
 
-class ServiceVMPlugin(vm_db.ServiceResourcePluginDb, ServiceVMMgmtMixin):
+class VNFMPlugin(vm_db.VNFMPluginDb, VNFMMgmtMixin):
     """ServiceVMPlugin which supports ServiceVM framework
     """
     OPTS = [
@@ -161,15 +163,16 @@ class ServiceVMPlugin(vm_db.ServiceResourcePluginDb, ServiceVMMgmtMixin):
             help=_('Hosting device drivers servicevm plugin will use')),
     ]
     cfg.CONF.register_opts(OPTS, 'servicevm')
-    supported_extension_aliases = ['servicevm']
+    supported_extension_aliases = ['vnfm']
 
     def __init__(self):
-        super(ServiceVMPlugin, self).__init__()
+        super(VNFMPlugin, self).__init__()
         self._pool = eventlet.GreenPool()
+        self.boot_wait = cfg.CONF.servicevm.boot_wait
         self._device_manager = driver_manager.DriverManager(
             'tacker.servicevm.device.drivers',
             cfg.CONF.servicevm.infra_driver)
-        self._device_status = monitor.DeviceStatus()
+        self._vnf_monitor = monitor.VNFMonitor(self.boot_wait)
 
     def spawn_n(self, function, *args, **kwargs):
         self._pool.spawn_n(function, *args, **kwargs)
@@ -184,18 +187,18 @@ class ServiceVMPlugin(vm_db.ServiceResourcePluginDb, ServiceVMMgmtMixin):
         infra_driver = template.get('infra_driver')
         if not attributes.is_attr_set(infra_driver):
             LOG.debug(_('hosting device driver must be specified'))
-            raise servicevm.InfraDriverNotSpecified()
+            raise vnfm.InfraDriverNotSpecified()
         if infra_driver not in self._device_manager:
             LOG.debug(_('unknown hosting device driver '
                         '%(infra_driver)s in %(drivers)s'),
                       {'infra_driver': infra_driver,
                        'drivers': cfg.CONF.servicevm.infra_driver})
-            raise servicevm.InvalidInfraDriver(infra_driver=infra_driver)
+            raise vnfm.InvalidInfraDriver(infra_driver=infra_driver)
 
         service_types = template.get('service_types')
         if not attributes.is_attr_set(service_types):
             LOG.debug(_('service type must be specified'))
-            raise servicevm.ServiceTypesNotSpecified()
+            raise vnfm.ServiceTypesNotSpecified()
         for service_type in service_types:
             # TODO(yamahata):
             # framework doesn't know what services are valid for now.
@@ -206,40 +209,31 @@ class ServiceVMPlugin(vm_db.ServiceResourcePluginDb, ServiceVMMgmtMixin):
             infra_driver, 'create_device_template_pre', plugin=self,
             context=context, device_template=device_template)
 
-        return super(ServiceVMPlugin, self).create_device_template(
+        return super(VNFMPlugin, self).create_device_template(
             context, device_template)
 
     ###########################################################################
     # hosting device
-
     def add_device_to_monitor(self, device_dict):
-        device_id = device_dict['id']
         dev_attrs = device_dict['attributes']
-        if dev_attrs.get('monitoring_policy') == 'ping':
-            def down_cb(hosting_device_):
-                if self._mark_device_dead(device_id):
-                    self._device_status.mark_dead(device_id)
-                    device_dict_ = self.get_device(
-                        t_context.get_admin_context(), device_id)
-                    failure_cls = monitor.FailurePolicy.get_policy(
-                        device_dict_['attributes'].get('failure_policy'),
-                        device_dict_)
-                    if failure_cls:
-                        failure_cls.on_failure(self, device_dict_)
 
-            hosting_device = self._device_status.to_hosting_device(
-                device_dict, down_cb)
-            KEY_LIST = ('monitoring_policy', 'failure_policy')
-            for key in KEY_LIST:
-                if key in dev_attrs:
-                    hosting_device[key] = dev_attrs[key]
-            self._device_status.add_hosting_device(hosting_device)
+        if 'monitoring_policy' in dev_attrs:
+            def action_cb(hosting_vnf_, action):
+                action_cls = monitor.ActionPolicy.get_policy(action,
+                    device_dict)
+                if action_cls:
+                    action_cls.execute_action(self, hosting_vnf['device'])
+
+            hosting_vnf = self._vnf_monitor.to_hosting_vnf(
+                device_dict, action_cb)
+            LOG.debug('hosting_vnf: %s', hosting_vnf)
+            self._vnf_monitor.add_hosting_vnf(hosting_vnf)
 
     def config_device(self, context, device_dict):
         config = device_dict['attributes'].get('config')
         if not config:
             return
-        eventlet.sleep(30)      # wait for vm to be ready
+        eventlet.sleep(self.boot_wait)      # wait for vm to be ready
         device_id = device_dict['id']
         update = {
             'device': {
@@ -253,16 +247,17 @@ class ServiceVMPlugin(vm_db.ServiceResourcePluginDb, ServiceVMMgmtMixin):
         driver_name = self._infra_driver_name(device_dict)
         device_id = device_dict['id']
         instance_id = self._instance_id(device_dict)
+        create_failed = False
 
         try:
             self._device_manager.invoke(
                 driver_name, 'create_wait', plugin=self, context=context,
                 device_dict=device_dict, device_id=instance_id)
-        except servicevm.DeviceCreateWaitFailed:
-            instance_id = None
-            del device_dict['instance_id']
+        except vnfm.DeviceCreateWaitFailed:
+            create_failed = True
+            device_dict['status'] = constants.ERROR
 
-        if instance_id is None:
+        if instance_id is None or create_failed:
             mgmt_url = None
         else:
             # mgmt_url = self.mgmt_url(context, device_dict)
@@ -271,11 +266,11 @@ class ServiceVMPlugin(vm_db.ServiceResourcePluginDb, ServiceVMMgmtMixin):
 
         self._create_device_post(
             context, device_id, instance_id, mgmt_url, device_dict)
-        if instance_id is None:
-            self.mgmt_create_post(context, device_dict)
+        self.mgmt_create_post(context, device_dict)
+
+        if instance_id is None or create_failed:
             return
 
-        self.mgmt_create_post(context, device_dict)
         device_dict['mgmt_url'] = mgmt_url
 
         kwargs = {
@@ -390,7 +385,7 @@ class ServiceVMPlugin(vm_db.ServiceResourcePluginDb, ServiceVMMgmtMixin):
 
     def delete_device(self, context, device_id):
         device_dict = self._delete_device_pre(context, device_id)
-        self._device_status.delete_hosting_device(device_id)
+        self._vnf_monitor.delete_hosting_vnf(device_id)
         driver_name = self._infra_driver_name(device_dict)
         instance_id = self._instance_id(device_dict)
 
@@ -415,27 +410,6 @@ class ServiceVMPlugin(vm_db.ServiceResourcePluginDb, ServiceVMMgmtMixin):
         self._delete_device_post(context, device_id, None)
         self.spawn_n(self._delete_device_wait, context, device_dict)
 
-    def _do_interface(self, context, device_id, port_id, action):
-        device_dict = self._update_device_pre(context, device_id)
-        driver_name = self._infra_driver_name(device_dict)
-        instance_id = self._instance_id(device_dict)
-
-        try:
-            self._device_manager.invoke(driver_name, action, plugin=self,
-                                        context=context, device_id=instance_id)
-        except Exception:
-            with excutils.save_and_reraise_exception():
-                device_dict['status'] = constants.ERROR
-                self._update_device_post(context, device_id, constants.ERROR)
-
-        self._update_device_post(context, device_dict['id'], constants.ACTIVE)
-
-    def attach_interface(self, context, id, port_id):
-        return self._do_interface(context, id, port_id, 'attach_interface')
-
-    def detach_interface(self, context, id, port_id):
-        return self._do_interface(context, id, port_id, 'dettach_interface')
-
     ###########################################################################
     # logical service instance
     #
@@ -473,7 +447,7 @@ class ServiceVMPlugin(vm_db.ServiceResourcePluginDb, ServiceVMMgmtMixin):
 
     def _create_service_instance_db(self, context, device_id,
                                     service_instance_param, managed_by_user):
-        return super(ServiceVMPlugin, self)._create_service_instance(
+        return super(VNFMPlugin, self)._create_service_instance(
             context, device_id, service_instance_param, managed_by_user)
 
     def _create_service_instance_by_type(
@@ -683,3 +657,25 @@ class ServiceVMPlugin(vm_db.ServiceResourcePluginDb, ServiceVMMgmtMixin):
         self.spawn_n(
             self._delete_service_instance_wait, context, device,
             service_instance, {}, None, None)
+
+    def create_vnf(self, context, vnf):
+        vnf['device'] = vnf.pop('vnf')
+        vnf_attributes = vnf['device']
+        vnf_attributes['template_id'] = vnf_attributes.pop('vnfd_id')
+        vnf_dict = self.create_device(context, vnf)
+        vnf_response = copy.deepcopy(vnf_dict)
+        vnf_response['vnfd_id'] = vnf_response.pop('template_id')
+        return vnf_response
+
+    def update_vnf(
+            self, context, vnf_id, vnf):
+        vnf['device'] = vnf.pop('vnf')
+        return self.update_device(context, vnf_id, vnf)
+
+    def delete_vnf(self, context, vnf_id):
+        self.delete_device(context, vnf_id)
+
+    def create_vnfd(self, context, vnfd):
+        vnfd['device_template'] = vnfd.pop('vnfd')
+        new_dict = self.create_device_template(context, vnfd)
+        return new_dict
